																																					/* 
 This sample introduces how to add ASP DataGrid Columns that contain combobox, dropdown
 column,  DatePicker, TextBox, CheckBox, UpDown column, TextFractions column,  TextEdit
 column,  Numeric column, Time column into an  ASP DataGrid server control  on your ASP 
 .NET Page.

 RustemSoft ASP DataGrid Columns assembly contains the special  DataGridCommands  class  
 that helps you manage a DataGrid datasource.  It allows you to input, update and store 
 data into rows in a currently opened datasource.  RustemSoft tried to  approximate the 
 Windows Forms DataGrid implementation to ASP.NET DataGrid server control and we got an 
 amazing result. By using UpdateDataSource method of the class  you can easily save all 
 updates that your user made in ASP DataGrid interface into bound datasource.

 The combobox column is not a dropdown list, which appears when a datagrid cell becomes 
 the current  cell.  The ASP .NET DataGrid combobox  Column control  has  the following 
 attractive features:

    This DataGrid Column gives you ability to instantly update cell value with a really 
    simple  and friendly user interface. When you click the special button of the combo 
    a separate designed by you interface Page will be shown over the datagrid.  You can 
    implement a dictionary of all combobox possible values on this interface Page.

    When the Interface Page appears it can receive the selected combobox value that can
    be used on the Interface Page.  Also when you close the Interface Page  by choosing
    some  item it  retrieves into  selected  datagrids combobox cell  two  parameters: 
    ValueMembers value and DisplayMembers value.

    You can set its DataSource, DisplayMember,  and ValueMember to bind the combobox to
    a foreign table.

 Copyright (C) 2004 RustemSoft.com

 For more info: http:\\rustemsoft.com\ASPDataGridColumns.htm
 -------------------------------------------------------------------------------------                                                       */
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Data.OleDb;
using RustemSoft;

namespace ColumnsCS
{
	public class Cars : System.Web.UI.Page
	{
		// Set Connection String to Samples.mdb
		private string conStr = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + HttpContext.Current.Server.MapPath("Samples.mdb");
		protected System.Web.UI.WebControls.Button btnAdd;
		protected System.Web.UI.WebControls.Button btnSave;
		protected System.Web.UI.WebControls.Button btnReset;
		protected System.Web.UI.WebControls.Button btnRustemSoft;
		protected System.Web.UI.WebControls.Panel ToolBar;
		protected System.Web.UI.WebControls.DataGrid DataGrid1;
		// The tblCars object temporarily store the data source for
		// the DataGrid control while the page  is being  processed
		DataTable tblCars = new DataTable();
		DataTable tblCompanies = new DataTable();
					   
		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.btnAdd.Click += new System.EventHandler(this.btnAdd_Click);
			this.btnSave.Click += new System.EventHandler(this.btnSave_Click);
			this.btnReset.Click += new System.EventHandler(this.btnReset_Click);
			this.btnRustemSoft.Click += new System.EventHandler(this.btnRustemSoft_Click);
			this.Load += new System.EventHandler(this.Page_Load);
			this.DataGrid1.EditCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(this.DataGrid1_EditCommand);
		}
		#endregion

		private void Page_Load(object sender, System.EventArgs e)
		{
			// With a database, use and select query to retrieve the data. Because the
			// data source in this example is a virtual DataTable,  retrieve  the data 
			// from session state  if it exists otherwise,  create  the  data  source.
			GetSource();

			// The DataGrid control maintains state between posts to the server
			// it only needs to be bound to a data source the first time the page
			// is loaded or when the data source is updated.
			if (!IsPostBack) BindGrid();
		}

		private void GetSource()
		{
			// The data source is a DataTable that is stored in session state.
			// If the data source does not exist, create it otherwise, load the data.
			if (Session["SessionDataSource"] == null) 
			{     
			   // Create the sample source data
				string SQL = "SELECT * FROM Cars";
				OleDbDataAdapter DA = new System.Data.OleDb.OleDbDataAdapter(SQL, conStr);
				DataSet DS = new DataSet();
				DA.Fill(DS, "Cars");
				// Populate the DataTable with sample data
				tblCars = DS.Tables["Cars"];

				// Store the table in session state to persist its values between posts to the server
				Session["SessionDataSource"] = tblCars;
			}
			else {
				// Retrieve the sample data from session state
				tblCars = (DataTable)Session["SessionDataSource"];
			}
		}

		private void BindGrid()
		{
			// Set datagrid columns
			string SQL = "SELECT * FROM Companies"; 
			OleDbDataAdapter DA = new System.Data.OleDb.OleDbDataAdapter(SQL, conStr); 
			DA.Fill(tblCompanies); 

			// Define comboCompany variable as an object of ComboBoxColumn class
			// and assign it to the DataGrid1 first 'Company' column.
			ComboBoxColumn comboCompany = (ComboBoxColumn)DataGrid1.Columns[0]; 
			// Define Data source for Company column as tblCompanies DataTable object
			comboCompany.DataSource = tblCompanies; 
			// Define Display Member for Company column as "Name" field of tblCompanies
			comboCompany.DisplayMember = "Name"; 
			// Define Value Member for Company column as "PubID" field of tblCompanies
			comboCompany.ValueMember = "PubID"; 
			// Define Name of Interface Page that will be called by clicking on ComboBox button
			comboCompany.InterfacePageName = "ComboInterfacePage.aspx"; 
			// Define width of Interface Page
			comboCompany.InterfacePageWidth = 345; 
			// Define height of Interface Page
			comboCompany.InterfacePageHeight = 400; 
			// Define if the Interface Page is scrollable
			comboCompany.InterfacePageScroll = true; 
			// Identify Company columns foreground color as dark blue
			comboCompany.ForeColor = Color.DarkBlue; 
			// Identify Company column characters font and size
			comboCompany.Font_Name = "Tahoma"; 
			comboCompany.Font_Size = FontUnit.Point(8); 
			// Adjust Company columns width
			comboCompany.Width = Unit.Point(110); 
			comboCompany.ReadOnlyBox = true; 

			// Define txbCar variable as an object of TextBoxColumn class
			// and assign it to the DataGrid1 second 'Car' column.
			TextBoxColumn txbCar = (TextBoxColumn)DataGrid1.Columns[1]; 
			// Identify Car columns foreground color as dark blue
			txbCar.ForeColor = Color.DarkBlue; 
			// Identify Car column characters font and size
			txbCar.Font_Name = "Tahoma"; 
			txbCar.Font_Size = FontUnit.Point(8); 
			// Adjust Car columns width
			txbCar.Width = Unit.Point(70); 

			// Define PhoneColumn variable as an object of TextFractionsColumn class
			// and assign it to the DataGrid1 'Phone' column.
			TextFractionsColumn PhoneColumn = (TextFractionsColumn)DataGrid1.Columns[2]; 
			// Identify 'Phone' columns background color
			PhoneColumn.BackColor = Color.LightGray;
			// Identify 'Phone' column characters font and size
			PhoneColumn.Font_Name = "Tahoma";
			PhoneColumn.Font_Size = FontUnit.Point(8);
			// Adjust 'Phone' columns width
			PhoneColumn.Width = Unit.Point(65);
			// Specify Delimiter Character for the field
			PhoneColumn.DelimiterChar = "-";
			// You may turn error messages off
			//PhoneColumn.ErrMessageVisible = false;
			// You can specify an error message in your native language:
			//PhoneColumn.ErrMessage = "Veuillez crire les nombres seulement!";

			// Specify first fraction properties
			// Alphanumeric symbols only are acceptable for the fraction
			PhoneColumn.FractionsCode[0, 0] = "a";
			// You can insert 3 symbols only into the first fraction
			PhoneColumn.FractionsCode[0, 1] = 3;

			// Specify second fraction properties
			// Numeric symbols only are acceptable for the fraction
			PhoneColumn.FractionsCode[1, 0] = "n";
			// You can insert 3 symbols only into the second fraction
			PhoneColumn.FractionsCode[1, 1] = 3;

			// Specify third fraction properties
			// Numeric symbols only are acceptable for the fraction
			PhoneColumn.FractionsCode[2, 0] = "n";
			// You can insert 4 symbols only into the third fraction
			PhoneColumn.FractionsCode[2, 1] = 4;

			// Define StateColumn variable as an object of UpDownColumn class
			// and assign it to the DataGrid1 'State' column.
			UpDownColumn StateColumn = (UpDownColumn)DataGrid1.Columns[3]; 
			ArrayList al = new ArrayList(); 
			SQL = "SELECT * FROM States"; 
			DA = new System.Data.OleDb.OleDbDataAdapter(SQL, conStr); 
			DataTable tblStates = new DataTable(); 
			DA.Fill(tblStates); 
			// Populate tblStates table's records into al ArrayList
			foreach (DataRow row in tblStates.Rows) 
			{ 
				al.Add(row["State"]); 
			} 
			StateColumn.DataSource = al; 
			// Identify State columns background color
			StateColumn.BackColor = Color.LightGray; 
			// Identify State column characters font and size
			StateColumn.Font_Name = "Tahoma"; 
			StateColumn.Font_Size = FontUnit.Point(8); 
			// Adjust State columns width
			StateColumn.Width = Unit.Point(30); 

			// Define CommentColumn variable as an object of TextEditColumn class
			// and assign it to the DataGrid1 'Comments' column.
			TextEditColumn CommentColumn = (TextEditColumn)DataGrid1.Columns[4]; 
			// Identify Comments columns background color
			CommentColumn.BackColor = Color.LightGray; 
			// Identify Comments column characters font and size
			CommentColumn.Font_Name = "Tahoma"; 
			CommentColumn.Font_Size = FontUnit.Point(8); 
			// Adjust Comments columns width
			CommentColumn.Width = Unit.Point(60); 
			// Define Name of Interface Page that will be called by clicking on TextEdit button
			CommentColumn.InterfacePageName = "TextEditPage.aspx";
			// Define width of Interface Page
			CommentColumn.InterfacePageWidth = 270;
			// Define height of Interface Page
			CommentColumn.InterfacePageHeight = 240;
			// Define if the Interface Page is scrollable
			CommentColumn.InterfacePageScroll = true;

			// Define dpDate variable as an object of DatePickerColumn class
			// and assign it to the DataGrid1 'Date' column.
			DatePickerColumn dpDate = (DatePickerColumn)DataGrid1.Columns[5]; 
			// Define Date Format
			DatePickerColumn.DateFormats dpFormat = DatePickerColumn.DateFormats.MMDDYYYY;
			dpDate.Format = dpFormat;
			// Identify Date columns background color
			dpDate.ForeColor = Color.DarkMagenta; 
			// Identify Date column characters font and size
			dpDate.Font_Name = "Tahoma"; 
			dpDate.Font_Size = FontUnit.Point(8); 
			// Adjust Date columns width
			dpDate.Width = Unit.Point(60); 
	        dpDate.yearGreaterThanPresentYear = 20;

			// Define tcTime variable as an object of TimeColumn class
			// and assign it to the DataGrid1 'TimeFirst' column.
			DateTimeColumn tcTime = (DateTimeColumn)DataGrid1.Columns[6];
			// Define Time Format
			DateTimeColumn.DateTimeFormats dtFormat = DateTimeColumn.DateTimeFormats.HHMMSS12;
			tcTime.Format = dtFormat;
			// Define separator char
			tcTime.DelimiterChar = ":";
			//tcTime.ErrMessageVisible = False;
			//tcTime.ErrMessage = "Date Inadmissible ! Rintroduisez svp.";
			// Identify Time columns foreground color
			tcTime.ForeColor = Color.DarkBlue;
			// Identify Time columns background color
			tcTime.BackColor = Color.Azure;
			// Identify Time column characters font and size
			tcTime.Font_Name = "Tahoma";
			tcTime.Font_Size = FontUnit.Point(8);
			// Adjust Time columns width
			tcTime.Width = Unit.Point(70);
			// Adjust 'AM/PM' Indicator width
			tcTime.IndicatorWidth = 30;

			// one more State field that shows State Name
			// Define ddStateID variable as an object of DropDownColumn class
			// and assign it to the DataGrid1 'StateID' column.
			DropDownColumn ddStateID = (DropDownColumn)DataGrid1.Columns[7]; 
			// Assign DataSource property for ddStateID object as tblStates
			// data table where statess names are stored.
			ddStateID.DataSource = tblStates; 
			// Specify DisplayMember as "Name" field of tblStates table
			ddStateID.DisplayMember = "Name"; 
			// Specify ValueMember as "State" field of tblStates table
			ddStateID.ValueMember = "State"; 
			// Identify StateID columns background color
			ddStateID.BackColor = Color.LightGray; 
			// Identify StateID column characters font and size
			ddStateID.Font_Name = "Tahoma"; 
			ddStateID.Font_Size = FontUnit.Point(8); 
			// Adjust StateID columns width
			ddStateID.Width = Unit.Point(80); 

			// Define Percent variable as an object of NumericColumn class
			// and assign it to the DataGrid1 'Percent' column.
			NumericColumn Percent = (NumericColumn)DataGrid1.Columns[8]; 
			// Identify Percent columns foreground color
			Percent.ForeColor = Color.DarkMagenta; 
			// Identify Percent columns background color
			Percent.BackColor = Color.LightCyan; 
			// Identify Percent column characters font and size
			Percent.Font_Name = "Tahoma"; 
			Percent.Font_Size = FontUnit.Point(8); 
			// Adjust Percent columns width
			Percent.Width = Unit.Point(30); 
			// Define that Percent column accepts positive values only
			Percent.PositiveOnly = true;
			// Define Percent columns max length
			Percent.MaxLength = 8;
			// Define Percent columns Decimal Length
			Percent.DecimalLength = 2;
			// Define Percent columns max possible value
			Percent.MaxValue = 10000;
			// Define Percent columns min possible value
			Percent.MinValue = 100;
			// Define Percent columns error message for max value
			Percent.ErrMessageMaxValue = "Too much!";

	        // Set the data source and bind to the Data Grid control
			DataGrid1.DataSource = tblCars; 
			DataGrid1.DataBind();
		}

		private void btnAdd_Click(object sender, System.EventArgs e)
		{
			// Add a new record to source tblCars table
			tblCars.Rows.Add(tblCars.NewRow());
			tblCars.AcceptChanges();
			BindGrid();
		}

		private void btnSave_Click(object sender, System.EventArgs e)
		{
			// Save datagrid data source tblCars table
			DataGridCommands DGCommands = new DataGridCommands();
			DGCommands.UpdateDataSource(tblCars, DataGrid1);
			BindGrid();

			// Save changes into Samples.mdb
			string SQLStr, Value;
			int i;
			System.Data.OleDb.OleDbConnection cn = new System.Data.OleDb.OleDbConnection(conStr);
			cn.Open();
			
			System.Data.OleDb.OleDbCommand cmd;

			// Delete previous data from Cars table
			try
			{
				SQLStr = "Delete from Cars";
				// Identify OleDb Command
				cmd = new System.Data.OleDb.OleDbCommand(SQLStr, cn);
				// Execute OleDb Command
				cmd.ExecuteNonQuery();
				cmd.Cancel();
			}
			catch (Exception er) 
			{
	            cn.Close();
				throw new Exception("Cannot update Samples.mdb! " + er.Message);
			}

			// Write updated data to Cars table
			try
			{
				// Accept Changes for Cars table
				// Go row by row in Cars table
				foreach(DataRow myRow in tblCars.Rows) 
				{
					// Combine SQL command
					SQLStr = "Insert into Cars values (";
					i = 0;
					// Go column by column for each row in Cars table
					foreach(DataColumn myCol in tblCars.Columns) 
					{
						Value = myRow[i].ToString();
						if (Value == String.Empty) 
						{
							Value = "Null,";
						}
						else 
						{
							if (myCol.DataType.ToString().Substring(7) == "Double") 
							{
								Value = myRow[i].ToString() + ",";
							}
							else if (myCol.DataType.ToString().Substring(7) == "Boolean") 
							{
								if (myRow[i].Equals(true)) 
								{
									Value = "-1,";
								}
								else
								{
									Value = "0,";
								}
							}
							else 
							{
								Value = "'" + myRow[i].ToString() + "',";
							}
						}
						SQLStr += Value;
						i += 1;
					}
					SQLStr = SQLStr.Substring(0, SQLStr.Length - 1) + ")";
					// Identify OleDb Command
					cmd = new System.Data.OleDb.OleDbCommand(SQLStr, cn);
					// Execute OleDb Command
					cmd.ExecuteNonQuery();
				}
			}
			catch (Exception er) 
			{
				cn.Close();
				throw new Exception("Cannot update Samples.mdb! " + er.Message);
			}
			cn.Close();
		}

		private void btnReset_Click(object sender, System.EventArgs e)
		{
			// Reset current updates
			BindGrid();
		}

		private void btnRustemSoft_Click(object sender, System.EventArgs e)
		{
	        Response.Redirect("http://rustemsoft.com");
		}

		private void DataGrid1_EditCommand(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e)
		{
			// Delete selected record from data source tblCars table
			DataGridCommands DGCommands = new DataGridCommands();
			DGCommands.DeleteRow(tblCars, e);
			BindGrid();
		}

	}
}























