' This sample introduces how to add ASP DataGrid Columns that contain combobox, dropdown
' column,  DatePicker, TextBox, CheckBox, UpDown column, TextFractions column,  TextEdit
' column,  Numeric column, Time column into an  ASP DataGrid server control  on your ASP 
' .NET Page.
'
' RustemSoft ASP DataGrid Columns assembly contains the special  DataGridCommands  class  
' that helps you manage a DataGrid datasource.  It allows you to input, update and store 
' data into rows in a currently opened datasource.  RustemSoft tried to  approximate the 
' Windows Forms DataGrid implementation to ASP.NET DataGrid server control and we got an 
' amazing result. By using UpdateDataSource method of the class  you can easily save all 
' updates that your user made in ASP DataGrid interface into bound datasource.
'
' The combobox column is not a dropdown list, which appears when a datagrid cell becomes 
' the current  cell.  The ASP .NET DataGrid combobox  Column control  has  the following 
' attractive features:
'
'   This DataGrid Column gives you ability to instantly update cell value with a really 
'   simple  and friendly user interface. When you click the special button of the combo 
'   a separate designed by you interface Page will be shown over the datagrid.  You can 
'   implement a dictionary of all combobox possible values on this interface Page.
'
'   When the Interface Page appears it can receive the selected combobox value that can
'   be used on the Interface Page.  Also when you close the Interface Page  by choosing
'   some  item it  retrieves into  selected  datagrids combobox cell  two  parameters: 
'   ValueMembers value and DisplayMembers value.
'
'   You can set its DataSource, DisplayMember,  and ValueMember to bind the combobox to
'   a foreign table.
'
' Copyright (C) 2004 RustemSoft.com
'
' For more info: http:\\rustemsoft.com\ASPDataGridColumns.htm
'---------------------------------------------------------------------------------------
Imports System.Data
Imports System.Data.OleDb
Imports RustemSoft

Public Class Cars
    Inherits System.Web.UI.Page

    Protected WithEvents DataGrid1 As System.Web.UI.WebControls.DataGrid
    Protected WithEvents btnSave As System.Web.UI.WebControls.Button
    ' Set Connection String to Samples.mdb
    Private conStr As String = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + Server.MapPath("Samples.mdb")
    Protected WithEvents btnReset As System.Web.UI.WebControls.Button
    Protected WithEvents btnRustemSoft As System.Web.UI.WebControls.Button
    Protected WithEvents btnAdd As System.Web.UI.WebControls.Button
    Protected WithEvents ToolBar As System.Web.UI.WebControls.Panel
    ' The tblCars object temporarily store the data source for
    ' the DataGrid control while the page  is being  processed
    Private tblCars As DataTable = New DataTable
    Private tblCompanies As DataTable = New DataTable

#Region " Web Form Designer Generated Code "

    'This call is required by the Web Form Designer.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()

    End Sub

    Private Sub Page_Init(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Init
        'CODEGEN: This method call is required by the Web Form Designer
        'Do not modify it using the code editor.
        InitializeComponent()
    End Sub

#End Region

    Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' With a database, use and select query to retrieve the data. Because the
        ' data source in this example is a virtual DataTable,  retrieve  the data 
        ' from session state if it exists  otherwise,  create  the  data  source.
        GetSource()

        ' The DataGrid control maintains state between posts to the server
        ' it only needs to be bound to a data source the first time the page
        ' is loaded or when the data source is updated.
        If Not IsPostBack Then BindGrid()
    End Sub

    Private Sub GetSource()
        ' The data source is a DataTable that is stored in session state.
        ' If the data source does not exist, create it otherwise, load the data.
        If IsNothing(Session("SessionDataSource")) Then
            ' Create the sample source data
            Dim SQL As String = "SELECT * FROM Cars"
            Dim DA As OleDbDataAdapter = New System.Data.OleDb.OleDbDataAdapter(SQL, conStr)
            Dim DS As DataSet = New DataSet
            DA.Fill(DS, "Cars")
            ' Populate the DataTable with sample data
            tblCars = DS.Tables("Cars")

            ' Store the table in session state to persist its values between posts to the server
            Session("SessionDataSource") = tblCars
        Else
            ' Retrieve the sample data from session state
            tblCars = CType(Session("SessionDataSource"), DataTable)
        End If
    End Sub

    Private Sub BindGrid()
        ' Set datagrid columns
        Dim SQL As String = "SELECT * FROM Companies"
        Dim DA As OleDbDataAdapter = New System.Data.OleDb.OleDbDataAdapter(SQL, conStr)
        DA.Fill(tblCompanies)

        ' Define comboCompany variable as an object of ComboBoxColumn class
        ' and assign it to the DataGrid1 first 'Company' column.
        Dim comboCompany As ComboBoxColumn = DataGrid1.Columns(0)
        ' Define Data source for Company column as tblCompanies DataTable object
        comboCompany.DataSource = tblCompanies
        ' Define Display Member for Company column as "Name" field of tblCompanies
        comboCompany.DisplayMember = "Name"
        ' Define Value Member for Company column as "PubID" field of tblCompanies
        comboCompany.ValueMember = "PubID"
        ' Define Name of Interface Page that will be called by clicking on ComboBox button
        comboCompany.InterfacePageName = "ComboInterfacePage.aspx"
        ' Define width of Interface Page
        comboCompany.InterfacePageWidth = 290
        ' Define height of Interface Page
        comboCompany.InterfacePageHeight = 400
        ' Define if the Interface Page is scrollable
        comboCompany.InterfacePageScroll = True
        ' Identify Company columns foreground color as dark blue
        comboCompany.ForeColor = Color.DarkBlue
        ' Identify Company column characters font and size
        comboCompany.Font_Name = "Tahoma"
        comboCompany.Font_Size = FontUnit.Point(8)
        ' Adjust Company columns width
        comboCompany.Width = Unit.Point(100)
        comboCompany.ReadOnlyBox = True

        ' Define txbCar variable as an object of TextBoxColumn class
        ' and assign it to the DataGrid1 second 'Car' column.
        Dim txbCar As TextBoxColumn = DataGrid1.Columns(1)
        ' Identify Car columns foreground color as dark blue
        txbCar.ForeColor = Color.DarkBlue
        ' Identify Car column characters font and size
        txbCar.Font_Name = "Tahoma"
        txbCar.Font_Size = FontUnit.Point(8)
        ' Adjust Car columns width
        txbCar.Width = Unit.Point(70)

        ' Define PhoneColumn variable as an object of TextFractionsColumn class
        ' and assign it to the DataGrid1 'Phone' column.
        Dim PhoneColumn As TextFractionsColumn = DataGrid1.Columns(2)
        ' Identify 'Phone' columns background color
        PhoneColumn.BackColor = Color.LightGray
        ' Identify 'Phone' column characters font and size
        PhoneColumn.Font_Name = "Tahoma"
        PhoneColumn.Font_Size = FontUnit.Point(8)
        ' Adjust 'Phone' columns width
        PhoneColumn.Width = Unit.Point(65)
        ' Specify Delimiter Character for the field
        PhoneColumn.DelimiterChar = "-"
        ' You may turn error messages off
        'PhoneColumn.ErrMessageVisible = False
        ' You can specify an error message in your native language:
        'PhoneColumn.ErrMessage = "Veuillez crire les nombres seulement!"

        ' Specify first fraction properties
        ' Alphanumeric symbols only are acceptable for the fraction
        PhoneColumn.FractionsCode(0, 0) = "a"
        ' You can insert 3 symbols only into the first fraction
        PhoneColumn.FractionsCode(0, 1) = 3

        ' Specify second fraction properties
        ' Numeric symbols only are acceptable for the fraction
        PhoneColumn.FractionsCode(1, 0) = "n"
        ' You can insert 3 symbols only into the second fraction
        PhoneColumn.FractionsCode(1, 1) = 3

        ' Specify third fraction properties
        ' Numeric symbols only are acceptable for the fraction
        PhoneColumn.FractionsCode(2, 0) = "n"
        ' You can insert 4 symbols only into the third fraction
        PhoneColumn.FractionsCode(2, 1) = 4

        ' Define StateColumn variable as an object of UpDownColumn class
        ' and assign it to the DataGrid1 'State' column.
        Dim StateColumn As UpDownColumn = DataGrid1.Columns(3)
        Dim al As ArrayList = New ArrayList
        SQL = "SELECT * FROM States"
        DA = New System.Data.OleDb.OleDbDataAdapter(SQL, conStr)
        Dim tblStates As DataTable = New DataTable
        DA.Fill(tblStates)
        Dim row As DataRow
		' Populate tblStates table's records into al ArrayList
        For Each row In tblStates.Rows
            al.Add(row("State"))
        Next
        StateColumn.DataSource = al
        ' Identify State columns background color
        StateColumn.BackColor = Color.LightGray
        ' Identify State column characters font and size
        StateColumn.Font_Name = "Tahoma"
        StateColumn.Font_Size = FontUnit.Point(8)
        ' Adjust State columns width
        StateColumn.Width = Unit.Point(30)

        ' Define CommentColumn variable as an object of TextEditColumn class
        ' and assign it to the DataGrid1 'Comments' column.
        Dim CommentColumn As TextEditColumn = DataGrid1.Columns(4)
        ' Identify Comment columns background color
        CommentColumn.BackColor = Color.LightGray
        ' Identify Comment column characters font and size
        CommentColumn.Font_Name = "Tahoma"
        CommentColumn.Font_Size = FontUnit.Point(8)
        ' Adjust Comment columns width
        CommentColumn.Width = Unit.Point(60)
        ' Define Name of Interface Page that will be called by clicking on TextEdit button
        CommentColumn.InterfacePageName = "TextEditPage.aspx"
        ' Define width of Interface Page
        CommentColumn.InterfacePageWidth = 270
        ' Define height of Interface Page
        CommentColumn.InterfacePageHeight = 240
        ' Define if the Interface Page is scrollable
        CommentColumn.InterfacePageScroll = True

        ' Define dpDate variable as an object of DatePickerColumn class
        ' and assign it to the DataGrid1 'DateFirst' column.
        Dim dpDate As DatePickerColumn = DataGrid1.Columns(5)
        ' Identify Date columns foreground color
        dpDate.ForeColor = Color.DarkMagenta
        ' Identify Date column characters font and size
        dpDate.Font_Name = "Tahoma"
        dpDate.Font_Size = FontUnit.Point(8)
        ' Adjust Date columns width
        dpDate.Width = Unit.Point(60)
        dpDate.yearGreaterThanPresentYear = 20

        ' Define tcTime variable as an object of TimeColumn class
        ' and assign it to the DataGrid1 'TimeFirst' column.
        Dim tcTime As DateTimeColumn = DataGrid1.Columns(6)
        ' Define Time Format
        tcTime.Format = tcTime.DateTimeFormats.HHMMSS12
        ' Define separator char
        tcTime.DelimiterChar = ":"
        'tcTime.ErrMessageVisible = False
        'tcTime.ErrMessage = "Date Inadmissible ! Rintroduisez svp."
        ' Identify Time columns foreground color
        tcTime.ForeColor = Color.DarkBlue
        ' Identify Time columns background color
        tcTime.BackColor = Color.Azure
        ' Identify Time column characters font and size
        tcTime.Font_Name = "Tahoma"
        tcTime.Font_Size = FontUnit.Point(8)
        ' Adjust Time columns width
        tcTime.Width = Unit.Point(70)
        ' Adjust 'AM/PM' Indicator width
        tcTime.IndicatorWidth = 30

        ' one more State field that shows State Name
        ' Define ddStateID variable as an object of DropDownColumn class
        ' and assign it to the DataGrid1 'StateID' column.
        Dim ddStateID As DropDownColumn = DataGrid1.Columns(7)
        ' Assign DataSource property for ddStateID object as tblStates
        ' data table where statess names are stored.
        ddStateID.DataSource = tblStates
        ' Specify DisplayMember as "Name" field of tblStates table
        ddStateID.DisplayMember = "Name"
        ' Specify ValueMember as "State" field of tblStates table
        ddStateID.ValueMember = "State"
        ' Identify StateID columns background color
        ddStateID.BackColor = Color.LightGray
        ' Identify StateID column characters font and size
        ddStateID.Font_Name = "Tahoma"
        ddStateID.Font_Size = FontUnit.Point(8)
        ' Adjust StateID columns width
        ddStateID.Width = Unit.Point(80)

        ' Define Percent variable as an object of NumericColumn class
        ' and assign it to the DataGrid1 'Percent' column.
        Dim Percent As NumericColumn = DataGrid1.Columns(8)
        ' Identify Percent columns foreground color
        Percent.ForeColor = Color.DarkMagenta
        ' Identify Percent columns background color
        Percent.BackColor = Color.LightCyan
        ' Identify Percent column characters font and size
        Percent.Font_Name = "Tahoma"
        Percent.Font_Size = FontUnit.Point(8)
        ' Adjust Percent columns width
        Percent.Width = Unit.Point(30)
        ' Define that Percent column accepts positive values only
        Percent.PositiveOnly = True
        ' Define Percent columns max length
        Percent.MaxLength = 8
        ' Define Percent columns Decimal Length
        Percent.DecimalLength = 2
        ' Define Percent columns max possible value
        Percent.MaxValue = 10000
        ' Define Percent columns min possible value
        Percent.MinValue = 100
        ' Define Percent columns error message for max value
        Percent.ErrMessageMaxValue = "Too much!"

        ' Set the data source and bind to the Data Grid control
        DataGrid1.DataSource = tblCars
        DataGrid1.DataBind()

    End Sub

    Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click
        ' Save datagrid data source tblCars table
        Dim DGCommands As DataGridCommands = New DataGridCommands
        DGCommands.UpdateDataSource(tblCars, DataGrid1)
        BindGrid()

        ' Save changes into Samples.mdb
        Dim myRow As DataRow, myCol As DataColumn, SQLStr, Value As String, i As Integer
        Dim cn As System.Data.OleDb.OleDbConnection = New System.Data.OleDb.OleDbConnection(conStr)
        cn.Open()

        Dim cmd As System.Data.OleDb.OleDbCommand

        ' Delete previous data from Cars table
        Try
            SQLStr = "Delete from Cars"
            ' Identify OleDb Command
            cmd = New System.Data.OleDb.OleDbCommand(SQLStr, cn)
            ' Execute OleDb Command
            cmd.ExecuteNonQuery()
            cmd.Cancel()
        Catch er As Exception
            cn.Close()
            Throw New Exception("Cannot update Samples.mdb. " + er.Message)
        End Try

        ' Write updated data to Cars table
        Try
            ' Accept Changes for Cars table
            ' Go row by row in Cars table
            For Each myRow In tblCars.Rows
                ' Combine SQL command
                SQLStr = "Insert into Cars values ("
                i = 0
                ' Go column by column for each row in Cars table
                For Each myCol In tblCars.Columns
                    Value = myRow.Item(i).ToString
                    If Value Is String.Empty Then
                        Value = "Null,"
                    Else
                        If Mid(myCol.DataType.ToString, 8) = "Double" Then
                            Value = myRow.Item(i).ToString + ","
                        ElseIf Mid(myCol.DataType.ToString, 8) = "Boolean" Then
                            Value = IIf(myRow.Item(i), "-1", "0") + ","
                        Else
                            Value = "'" + myRow.Item(i).ToString + "',"
                        End If
                    End If
                    SQLStr += Value
                    i += 1
                Next
                SQLStr = Mid(SQLStr, 1, Len(SQLStr) - 1) + ")"
                ' Identify OleDb Command
                cmd = New System.Data.OleDb.OleDbCommand(SQLStr, cn)
                ' Execute OleDb Command
                cmd.ExecuteNonQuery()
            Next
        Catch er As Exception
            cn.Close()
            Throw New Exception("Cannot update Samples.mdb. " + er.Message)
        End Try
        cn.Close()

    End Sub

    Private Sub btnReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReset.Click
        ' Reset current updates
        BindGrid()
    End Sub

    Private Sub btnRustemSoft_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRustemSoft.Click
        Response.Redirect("http://rustemsoft.com")
    End Sub

    Private Sub btnAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAdd.Click
        ' Add a new record to source tblCars table
        tblCars.Rows.Add(tblCars.NewRow)
        tblCars.AcceptChanges()
        BindGrid()
    End Sub

    Private Sub DataGrid1_EditCommand(ByVal source As Object, ByVal e As System.Web.UI.WebControls.DataGridCommandEventArgs) Handles DataGrid1.EditCommand
        ' Delete selected record from data source tblCars table
        Dim DGCommands As DataGridCommands = New DataGridCommands
        DGCommands.DeleteRow(tblCars, e)
        BindGrid()
    End Sub

End Class
